from importlib import metadata
from platform import python_version

__version__ = "0.3.5"


def show_versions() -> None:
    msg = f"Versions\npython: {python_version()}\nspei: {__version__}\n"

    requirements = metadata.requires("spei")
    if requirements:
        deps = [x for x in requirements if "extra" not in x]
        for dep in deps:
            msg += f"{dep}: {metadata.version(dep)}\n"

    print(msg)



from typing import Optional, Union

from numpy import linspace, std
from pandas import DatetimeIndex, Series
from scipy.stats import fisk, gamma, genextreme, norm

from ._typing import ContinuousDist, NDArrayFloat
from .utils import validate_index, validate_series


def compute_si_ppf(
    series: Series,
    dist: ContinuousDist,
    index: Optional[DatetimeIndex] = None,
    sgi: bool = False,
    prob_zero: bool = False,
) -> Series:
    """Internal helper function to calculate drought index

    Parameters
    ----------
    series : Series
        Series with observations
    dist : ContinuousDist
        Continuous distribution from the SciPy library
    index : DatetimeIndex, optional
        DatetimeIndex with the date of the observations
    sgi : bool, optional
        Whether to caclulate the standardized groundwater index or not, by
        default False
    prob_zero : bool, optional
        Apply logic to observations that have value zero and calculate their
        probability seperately, by default False

    Returns
    -------
    Series
        Series with probability point function, ppf
    """

    if index is None:
        series = validate_series(series)
        index = validate_index(series.index)

    si = Series(index=index, dtype=float)  # type: Series
    for month in range(1, 13):
        data = series[index.month == month].sort_values()
        if not sgi:
            if prob_zero:
                cdf = compute_cdf_probzero(data=data, dist=dist)
            else:
                cdf = compute_cdf(data=data, dist=dist)
        else:
            cdf = compute_cdf_nsf(data=data)
        ppf = norm.ppf(cdf)
        si.loc[data.index] = ppf
    return si


def compute_cdf(
    data: Union[Series, NDArrayFloat], dist: ContinuousDist
) -> NDArrayFloat:
    *pars, loc, scale = dist.fit(data, scale=std(data))
    cdf = dist.cdf(data, pars, loc=loc, scale=scale)
    return cdf


def compute_cdf_probzero(
    data: Union[Series, NDArrayFloat], dist: ContinuousDist
) -> NDArrayFloat:
    p0 = (data == 0.0).sum() / len(data)
    *pars, loc, scale = dist.fit(data[data != 0.0], scale=std(data))
    cdf_sub = dist.cdf(data, pars, loc=loc, scale=scale)
    cdf = p0 + (1 - p0) * cdf_sub
    cdf[data == 0.0] = p0
    return cdf


def compute_cdf_nsf(data: Union[Series, NDArrayFloat]) -> NDArrayFloat:
    """Normal Scores Transform"""
    n = data.size
    cdf = linspace(1 / (2 * n), 1 - 1 / (2 * n), n)
    return cdf


def sgi(series: Series) -> Series:
    """Method to compute the Standardized Groundwater Index [sgi_2013]_.
    Same method as in Pastas.

    Parameters
    ----------
    series: pandas.Series
        Pandas time series of the groundwater levels. Time series index
        should be a pandas DatetimeIndex.

    Returns
    -------
    pandas.Series

    References
    ----------
    .. [sgi_2013] Bloomfield, J. P. and Marchant, B. P.: Analysis of
       groundwater drought building on the standardised precipitation index
       approach. Hydrol. Earth Syst. Sci., 17, 4769–4787, 2013.
    """
    mock_dist = norm  # not used
    return compute_si_ppf(series=series, dist=mock_dist, sgi=True)


def spi(
    series: Series, dist: ContinuousDist = gamma, prob_zero: bool = False
) -> Series:
    """Method to compute the Standardized Precipitation Index [spi_2002]_.

    Parameters
    ----------
    series: pandas.Series
        Pandas time series of the precipitation. Time series index
        should be a pandas DatetimeIndex.
    dist: scipy.stats.rv_continuous
        Can be any continuous distribution from the scipy.stats library.
        However, for the SPI generally the Gamma probability density
        function is recommended. Other appropriate choices could be the
        lognormal, log-logistic (fisk) or PearsonIII distribution.
    prob_zero: bool
        Option to correct the distribution if x=0 is not in probability
        density function. E.g. the case with the Gamma distriubtion.

    Returns
    -------
    pandas.Series

    References
    ----------
    .. [spi_2002] LLoyd-Hughes, B. and Saunders, M.A.: A drought
       climatology for Europe. International Journal of Climatology,
       22, 1571-1592, 2002.
    """

    return compute_si_ppf(series=series, dist=dist, prob_zero=prob_zero)


def spei(series: Series, dist: ContinuousDist = fisk) -> Series:
    """Method to compute the Standardized Precipitation Evaporation Index
    [spei_2010]_.

    Parameters
    ----------
    series: pandas.Series
        Pandas time series of the precipitation. Time series index
        should be a pandas DatetimeIndex.
    dist: scipy.stats.rv_continuous
        Can be any continuous distribution from the scipy.stats library.
        However, for the SPEI generally the log-logistic (fisk) probability
        density function is recommended. Other appropriate choices could be
        the lognormal or PearsonIII distribution.

    Returns
    -------
    pandas.Series

    References
    ----------
    .. [spei_2010] Vicente-Serrano S.M., Beguería S., López-Moreno J.I.:
       A Multi-scalar drought index sensitive to global warming: The
       Standardized Precipitation Evapotranspiration Index.
       Journal of Climate, 23, 1696-1718, 2010.
    """

    return compute_si_ppf(series=series, dist=dist)


def ssfi(series: Series, dist: Optional[ContinuousDist] = genextreme) -> Series:
    """Method to compute the Standardized StreamFlow Index [ssfi_2020]_.

    Parameters
    ----------
    series: pandas.Series
        Pandas time series of the precipitation. Time series index
        should be a pandas DatetimeIndex.
    dist: scipy.stats.rv_continuous
        Can be any continuous distribution from the scipy.stats library.
        However, for the SSFI generally the gamma probability density
        function is recommended. Other appropriate choices could be the
        normal, lognormal, pearsonIII, GEV or  Gen-Logistic distribution.

    Returns
    -------
    pandas.Series

    References
    ----------
    .. [ssfi_2020] Tijdeman, E., Stahl, K., & Tallaksen, L. M.:
       Drought characteristics derived based on the Standardized
       Streamflow Index: A large sample comparison for parametric
       and nonparametric methods. Water Resources Research, 56, 2020.
    """

    return compute_si_ppf(series=series, dist=dist)



# https://www.climdex.org/

from pandas import Series

from .utils import validate_index, validate_series


def rxnday(series: Series, interval: str, period: str = "30D") -> Series:
    """Maximum consecutive precipitation amount over an interval"""
    series = validate_series(series)
    _ = validate_index(series.index)

    return series.rolling(interval).sum().rolling(period).max()


def rx1day(series: Series, interval: str = "1D", period: str = "30D") -> Series:
    """Maximum 1-day precipitation amount"""
    return rxnday(series=series, interval=interval, period=period)


def rx5day(series: Series, interval: str = "5D", period: str = "30D") -> Series:
    """Maximum consecutive precipitation amount over an 5-day interval"""
    return rxnday(series=series, interval=interval, period=period)


def sdii(series: Series, threshold: float = 1.0, period: str = "30D") -> Series:
    """Simple precipitation intensity index"""
    series = validate_series(series)
    _ = validate_index(series.index)

    w = series >= threshold

    return series.loc[w].resample(period).sum() / w.sum()


def rnmm(series: Series, threshold: float, period: str = "1Y") -> Series:
    """Annual count of days when precipitation ≥ n mm. n is a user-defined threshold"""
    series = validate_series(series)
    _ = validate_index(series.index)

    w = series >= threshold

    return w.resample(period).sum()


def r10mm(series: Series, threshold: float = 10.0, period: str = "1Y") -> Series:
    """Annual count of days when precipitation ≥ 10 mm"""
    return rnmm(series=series, threshold=threshold, period=period)


def r20mm(series: Series, threshold: float = 20.0, period: str = "1Y") -> Series:
    """Annual count of days when precipitation ≥ 20 mm"""
    return rnmm(series=series, threshold=threshold, period=period)


def cdd(series: Series, threshold: float = 1.0, period: str = "365D") -> Series:
    """Maximum length of dry spell: maximum number of consecutive days with
    precipitation < 1mm"""
    series = validate_series(series)
    _ = validate_index(series.index)

    w = series < threshold

    return w.diff().rolling(period).sum().dropna().astype(int)


def cwd(series: Series, threshold: float = 1.0, period: str = "365D") -> Series:
    """Maximum length of wet spell: maximum number of consecutive days with
    precipitation ≥ 1mm"""
    series = validate_series(series)
    _ = validate_index(series.index)

    w = series >= threshold

    return w.diff().rolling(period).sum().dropna().astype(int)


def prcptot(series: Series, period: str = "1Y") -> Series:
    """Total precipitation on wet days over a certain period"""
    series = validate_series(series)
    _ = validate_index(series.index)

    return series.resample(period).sum()


def rnnp(
    series: Series, quantile: float, threshold: float = 1.0, period: str = "1Y"
) -> Series:
    """Total amount of precipitation on wet days above certain quantile"""
    series = validate_series(series)
    _ = validate_index(series.index)

    series_w = series[series >= threshold]
    wq = series_w > series_w.quantile(quantile)

    return series_w.loc[wq].resample(period).sum()


def r95p(
    series: Series, quantile: float = 0.95, threshold: float = 1.0, period: str = "1Y"
) -> Series:
    """Total amount of precipitation on very wet days"""
    return rnnp(series=series, quantile=quantile, threshold=threshold, period=period)


def r99p(
    series: Series, quantile: float = 0.99, threshold: float = 1.0, period: str = "1Y"
) -> Series:
    """Total amount of precipitation on extremely wet days"""
    return rnnp(series=series, quantile=quantile, threshold=threshold, period=period)


def r95ptot(
    series: Series, quantile: float = 0.95, threshold: float = 1.0, period: str = "1Y"
) -> Series:
    """Contribution to total precipitation from very wet days"""
    r95 = r95p(series=series, quantile=quantile, threshold=threshold, period=period)
    tot = prcptot(series=series, period=period)
    return r95 * 100 / tot


def r99ptot(
    series: Series, quantile: float = 0.99, threshold: float = 1.0, period: str = "1Y"
) -> Series:
    """Contribution to total precipitation from extremely wet days"""
    r99 = r99p(series=series, quantile=quantile, threshold=threshold, period=period)
    tot = prcptot(series=series, period=period)
    return r99 * 100 / tot



import matplotlib as mpl
from pandas import Series
from scipy.stats import pearson3

from spei.plot import dist, monthly_density
from spei.plot import si as plot_si

mpl.use("Agg")  # prevent _tkinter.TclError: Can't find a usable tk.tcl error


def test_plot_si(si: Series) -> None:
    _ = plot_si(si)


def test_plot_dist(prec: Series) -> None:
    series = prec.rolling("90D", min_periods=90).sum().dropna()
    _ = dist(series, dist=pearson3, test_dist=False)


def test_plot_dist_cum(prec: Series) -> None:
    series = prec.rolling("90D", min_periods=90).sum().dropna()
    _ = dist(series, dist=pearson3, cumulative=True, test_dist=False)


def test_plot_dist_test(prec: Series) -> None:
    series = prec.rolling("90D", min_periods=90).sum().dropna()
    _ = dist(series, dist=pearson3, test_dist=True, cmap="viridis")


def test_plot_monthly_density(si: Series) -> None:
    _ = monthly_density(si, years=[2011], months=[1, 2, 3, 4, 5])



from pandas import Series

from spei import sgi, spei, spi, ssfi


def test_spi(prec: Series) -> None:
    spi(prec.rolling("30D", min_periods=30).sum().dropna(), prob_zero=True)


def test_spei(prec: Series, evap: Series) -> None:
    n = (prec - evap).rolling("30D", min_periods=30).sum().dropna()
    spei(n)


def test_sgi(head: Series) -> None:
    sgi(head)


def test_sffi(prec: Series) -> None:
    sf = prec.rolling("30D", min_periods=30).sum().dropna()
    ssfi(sf)



import logging

import pytest
from pandas import DataFrame, Series, to_datetime

from spei.utils import validate_index, validate_series


def test_validate_index(caplog) -> None:
    caplog.set_level(logging.INFO)
    series = Series([1, 2, 3], index=["2018", "2019", "2020"])
    validate_index(series.index)
    msg = (
        f"Expected the index to be a DatetimeIndex. Automatically converted "
        f"{type(series.index)} using pd.to_datetime(Index)\n"
    )
    print(f"{caplog.text=}")
    assert msg in caplog.text


def test_validate_series() -> None:
    with pytest.raises(TypeError):
        validate_series([1, 2, 3])


def test_validate_series_df_1d(caplog) -> None:
    df = DataFrame({"s": [1, 2, 3]}, index=to_datetime([1, 2, 3]))
    validate_series(df)
    msg = (
        "Please convert series of type pandas.DataFrame to a"
        "pandas.Series using DataFrame.squeeze(). Now done automatically.\n"
    )
    assert msg in caplog.text


def test_validate_series_df_2d() -> None:
    with pytest.raises(TypeError):
        df = DataFrame({"s1": [1, 2, 3], "s2": [1, 2, 3]}, index=to_datetime([1, 2, 3]))
        validate_series(df)



import logging
from typing import List, Optional, Tuple, Union

from numpy import std
from pandas import DataFrame, DatetimeIndex, Index, Series, to_datetime
from scipy.stats import (
    fisk,
    gamma,
    genextreme,
    genlogistic,
    kstest,
    logistic,
    lognorm,
    norm,
    pearson3,
)

from ._typing import ContinuousDist, NDArrayFloat


def validate_series(series: Series) -> Series:
    series = series.copy()

    if not isinstance(series, Series):
        if isinstance(series, DataFrame):
            if len(series.columns) == 1:
                logging.warning(
                    "Please convert series of type pandas.DataFrame to a"
                    "pandas.Series using DataFrame.squeeze(). Now done automatically."
                )
                series = series.squeeze()
            else:
                raise TypeError(
                    "Please provide a pandas.Series instead of a pandas.DataFrame"
                )
        else:
            raise TypeError(f"Please provide a Pandas Series instead of {type(series)}")

    return series


def validate_index(index: Index) -> DatetimeIndex:
    index = index.copy()

    if not isinstance(index, DatetimeIndex):
        logging.info(
            f"Expected the index to be a DatetimeIndex. Automatically converted "
            f"{type(index)} using pd.to_datetime(Index)"
        )
        index = DatetimeIndex(to_datetime(index))

    return index


def dist_test(
    series: Union[Series, NDArrayFloat],
    dist: ContinuousDist,
    N: int = 100,
    alpha: float = 0.05,
) -> Tuple[str, float, bool, tuple]:
    """Fit a distribution and perform the two-sided
    Kolmogorov-Smirnov test for goodness of fit. The
    null hypothesis is that the data and distributions
    are identical, the alternative is that they are
    not identical. [scipy_2021]_

    Parameters
    ----------
    data : Union[Series, NDArray[float]]
        pandas Series or numpy array of floats of observations of random
        variables
    dist: scipy.stats.rv_continuous
        Can be any continuous distribution from the
        scipy.stats library.
    N : int, optional
        Sample size, by default 100
    alpha : float, optional
        Significance level for testing, default is 0.05
        which is equal to a a confidence level of 95%;
        that is, the null hypothesis will be rejected in
        favor of the alternative if the p-value is
        less than 0.05.

    Returns
    -------
    string, float, bool, tuple
        distribution name, p-value and fitted parameters

    References
    -------
    .. [scipy_2021] Onnen, H.: Intro to Probability
     Distributions and Distribution Fitting with Pythons
    SciPy, 2021.
    """
    fitted = dist.fit(series, scale=std(series))
    dist_name = getattr(dist, "name")
    ks = kstest(series, dist_name, fitted, N=N)[1]
    rej_h0 = ks < alpha
    return dist_name, ks, rej_h0, fitted


def dists_test(
    series: Union[Series, NDArrayFloat],
    distributions: Optional[List[ContinuousDist]] = None,
    N: int = 100,
    alpha: float = 0.05,
) -> DataFrame:
    """Fit a list of distribution and perform the
    two-sided Kolmogorov-Smirnov test for goodness
    of fit. The null hypothesis is that the data and
    distributions are identical, the alternative is
    that they are not identical. [scipy_2021]_

    Parameters
    ----------
    series : Union[Series, NDArray[float]]
        pandas Series with observations of random variables
    distributions : list of scipy.stats.rv_continuous, optional
        A list of (can be) any continuous distribution from the scipy.stats
        library, by default None which makes a custom selection
    N : int, optional
        Sample size, by default 100
    alpha : float, optional
        Significance level for testing, default is 0.05
        which is equal to a a confidence level of 95%;
        that is, the null hypothesis will be rejected in
        favor of the alternative if the p-value is
        less than 0.05.

    Returns
    -------
    pandas.DataFrame
        DataFrame with the distribution names,
        pvalues and parameters

    References
    -------
    .. [scipy_2021] Onnen, H.: Intro to Probability
     Distributions and Distribution Fitting with Pythons
    SciPy, 2021.
    """
    if distributions is None:
        distributions = [
            norm,
            gamma,
            genextreme,
            pearson3,
            fisk,
            lognorm,
            logistic,
            genlogistic,
        ]

    df = DataFrame([dist_test(series, D, N, alpha) for D in distributions])
    cols = ["Distribution", "KS p-value", "Reject H0"]
    cols += [f"Param {i+1}" for i in range(len(df.columns) - len(cols))]
    df = df.rename(columns=dict(zip(df.columns, cols))).set_index(cols[0])
    df["Dist"] = distributions

    return df



from calendar import month_abbr, month_name
from itertools import cycle
from typing import List, Optional

import matplotlib.pyplot as plt
from numpy import array, linspace, meshgrid, reshape
from pandas import Series
from scipy.stats import gaussian_kde

from ._typing import Axes, ContinuousDist, NDArrayAxes
from .utils import dist_test, validate_index, validate_series


def si(
    si: Series, bound: float = 3.0, figsize: tuple = (6.5, 4), ax: Optional[Axes] = None
) -> Axes:
    """Plot the standardized index values as a time series.

    Parameters
    ----------
    si : pandas.Series
        Series of the standardized index
    bound : int, optional
        Maximum and minimum ylim of plot
    figsize : tuple, optional
        Figure size, by default (8, 4)
    ax : matplotlib.Axes, optional
        Axes handle, by default None which create a new axes

    Returns
    -------
    matplotlib.Axes
        Axes handle
    """
    if ax is None:
        _, ax = plt.subplots(figsize=figsize)

    ax.plot(si.index, si.values, linewidth=1.0, color="k")
    ax.axhline(0, linestyle="--", linewidth=1.0, color="k")

    nmin = -bound
    nmax = bound
    droughts = si.to_numpy(dtype=float, copy=True)
    droughts[droughts > 0] = 0
    nodroughts = si.to_numpy(dtype=float, copy=True)
    nodroughts[nodroughts < 0] = 0

    x, y = meshgrid(si.index, linspace(nmin, nmax, 100))
    ax.contourf(
        x, y, y, cmap=plt.get_cmap("seismic_r"), levels=linspace(nmin, nmax, 100)
    )
    ax.fill_between(x=si.index, y1=droughts, y2=nmin, color="w")
    ax.fill_between(x=si.index, y1=nodroughts, y2=nmax, color="w")
    ax.set_ylim(nmin, nmax)

    return ax


def dist(
    series: Series,
    dist: ContinuousDist,
    cumulative: bool = False,
    test_dist: bool = True,
    cmap: Optional[str] = None,
    figsize: tuple = (6.5, 10),
    legend: bool = True,
) -> NDArrayAxes:
    """Plot the (cumulative) histogram and scipy fitted distribution
    for the time series on a monthly basis.

    Parameters
    ----------
    series : pandas.Series
        Time series of the precipitation (excess) or head
    dist : scipy.stats._continuous_distns
        Continuous distribution of the scipy stats library
        to fit on series using maximum likelihood estimation
    cumulative : bool, optional
        If True plots cumulative histogram instead of probability
        density histogram, by default False
    test_dist : bool, optional
        If True, fit the distribution with the two-sided
        Kolmogorov-Smirnov test for goodness of fit.
    cmap : str, optional
        Matplotlib colormap name to use in subplots, by default None
        which uses black for all subplots.
    figsize : tuple, optional
        Figure size, by default (8, 10)
    legend : bool, optional
        Add legend, by default True

    Returns
    -------
    matplotlib.Axes
        Axes handle
    """

    series = validate_series(series)
    index = validate_index(series.index)

    _, axs = plt.subplots(4, 3, figsize=figsize, sharey=True, sharex=True)
    if cmap is not None:
        cm = plt.get_cmap(cmap, 12)
        c = [cm(i) for i in range(12)]
    else:
        c = [(1, 1, 1, 1) for _ in range(12)]

    for i, ax in enumerate(axs.flat, start=1):
        data = series[index.month == i].sort_values()
        ax.hist(
            data,
            color=c[i - 1],
            alpha=0.2,
            density=True,
            cumulative=cumulative,
            label="Density",
        )
        dist_name = getattr(dist, "name")
        if test_dist:
            _, p_value, _, fitted = dist_test(data, dist)
            *pars, loc, scale = fitted
            label = f"{dist_name.capitalize()} KS:\n{p_value=:0.2f}"
        else:
            *pars, loc, scale = dist.fit(data, scale=data.std())
            label = f"{dist_name.capitalize()} fit:\n{loc=:0.1f}\n{scale=:0.1f}"
        if cumulative:
            cdf = dist.cdf(data, pars, loc=loc, scale=scale)
            ax.plot(
                data,
                cdf,
                color=c[i - 1],
                label=label,
            )
            if i in range(1, 13, 3):
                ax.set_ylabel("Cumulative Probability")
        else:
            x = linspace(min(data), max(data))
            pdf = dist.pdf(x, pars, loc=loc, scale=scale)
            ax.plot(
                x,
                pdf,
                color=c[i - 1],
                label=label,
            )
            if i in range(1, 13, 3):
                ax.set_ylabel("Probability Density")
        ax.set_title(month_name[i])
        if legend:
            ax.legend(fontsize=7)

    return axs


def monthly_density(
    si: Series,
    years: List[int],
    months: List[int],
    cmap: str = "tab20c",
    ax: Optional[Axes] = None,
) -> Axes:
    """Plot the monthly kernel-density estimate for a specific year.

    Parameters
    ----------
    si : pandas.Series
        Series of the standardized index
    year : list, optional
        List of years as int
    months : list, optional
        List of months as int, by default all months
    cmap : str, optional
        matlotlib colormap, by default 'tab10'
    ax : matplotlib.Axes, optional
        Axes handle, by default None which create a new axes

    Returns
    -------
    matplotlib.Axes
        Axes handle
    """

    si = validate_series(si)
    index = validate_index(si.index)

    if ax is None:
        _, ax = plt.subplots(figsize=(6.5, 4))

    cm = plt.get_cmap(cmap, 20)
    colors = reshape(array([cm(x) for x in range(20)], dtype="f,f,f,f"), (5, 4))
    lsts = cycle(["--", "-.", ":"])

    ind = linspace(-3.3, 3.3, 1000)
    for i, month in enumerate(months):
        gkde_all = gaussian_kde(si[(index.month == month)])
        ax.plot(
            ind,
            gkde_all.evaluate(ind),
            c=colors[i, 0],
            label=f"{month_abbr[month]} all",
        )
        for j, year in enumerate(years, start=1):
            gkde_spec = gaussian_kde(si[(index.month == month) & (index.year == year)])
            ax.plot(
                ind,
                gkde_spec.evaluate(ind),
                c=colors[i, j],
                label=f"{month_abbr[month]} {year}",
                linestyle=next(lsts),
            )
    ax.set_ylabel("Kernel-Density Estimate")
    ax.set_xlim(ind[0], ind[-1])
    ax.set_ylim(bottom=0)
    ax.legend()
    ax.grid(True)

    return ax



from pandas import Series

from spei import climdex


def text_climdex_rxnday(precmm: Series) -> None:
    climdex.rxnday(series=precmm, interval="10D", period="90D")


def text_climdex_rx1day(precmm: Series) -> None:
    climdex.rx1day(series=precmm)


def text_climdex_rx5day(precmm: Series) -> None:
    climdex.rx5day(series=precmm)


def test_climdex_sdii(precmm: Series) -> None:
    climdex.sdii(series=precmm)


def test_climdex_rnmm(precmm: Series) -> None:
    climdex.rnmm(series=precmm, threshold=5, period="90D")


def test_climdex_r10mm(precmm: Series) -> None:
    climdex.r10mm(series=precmm)


def test_climdex_r20mm(precmm: Series) -> None:
    climdex.r20mm(series=precmm)


def test_climdex_cdd(precmm: Series) -> None:
    climdex.cdd(series=precmm)


def test_climdex_cwd(precmm: Series) -> None:
    climdex.cwd(series=precmm)


def test_climdex_prcptot(precmm: Series) -> None:
    climdex.prcptot(series=precmm)


def test_climdex_rnnp(precmm: Series) -> None:
    climdex.rnnp(series=precmm, quantile=0.5)


def test_climdex_r95p(precmm: Series) -> None:
    climdex.r95p(series=precmm)


def test_climdex_r99p(precmm: Series) -> None:
    climdex.r99p(series=precmm)


def test_climdex_r95ptot(precmm: Series) -> None:
    climdex.r95ptot(series=precmm)


def test_climdex_r99ptot(precmm: Series) -> None:
    climdex.r99ptot(series=precmm)



# flake8: noqa
from typing import Any, Union

from numpy import float64
from numpy.typing import NDArray
from scipy.stats._continuous_distns import rv_continuous

ContinuousDist = Union[Any, rv_continuous]
Axes = Any
NDArrayAxes = NDArray[Axes]
NDArrayFloat = NDArray[float64]
